@@ -31,6 +31,7 @@
 #include <KTp/global-contact-manager.h>
 #include <KTp/actions.h>
 #include <KTp/contact.h>
+#include <KTp/Widgets/settings-kcm-dialog.h>
 
 #include <KGlobal>
 #include <KSharedConfig>
@@ -43,6 +44,7 @@
 #include <KSettings/Dialog>
 #include <KMenu>
 #include <KNotifyConfigWidget>
+#include <KPushButton>
 
 #include <QHeaderView>
 #include <QLabel>
@@ -101,7 +103,6 @@ ContactListWidget::ContactListWidget(QWidget *parent)
     d->compactDelegate = new ContactDelegateCompact(ContactDelegateCompact::Normal, this);
 
     d->model = new KTp::ContactsModel(this);
-    d->model->setTrackUnreadMessages(true);
     d->model->setDynamicSortFilter(true);
     d->model->setSortRole(Qt::DisplayRole);
     d->style.reset(new NoLinesStyle());
@@ -166,6 +167,11 @@ ContactListWidget::~ContactListWidget()
     delete d_ptr;
 }
 
+KTp::ContactsModel* ContactListWidget::contactsModel() const
+{
+    return d_ptr->model;
+}
+
 void ContactListWidget::setAccountManager(const Tp::AccountManagerPtr &accountManager)
 {
     Q_D(ContactListWidget);
@@ -201,38 +207,10 @@ void ContactListWidget::setAccountManager(const Tp::AccountManagerPtr &accountMa
 
 void ContactListWidget::showSettingsKCM()
 {
-    KSettings::Dialog *dialog = new KSettings::Dialog(this);
-
-    KService::Ptr tpAccKcm = KService::serviceByDesktopName("kcm_ktp_accounts");
-
-    if (!tpAccKcm) {
-        KMessageBox::error(this,
-                           i18n("It appears you do not have the IM Accounts control module installed. Please install ktp-accounts-kcm package."),
-                           i18n("IM Accounts KCM Plugin Is Not Installed"));
-    }
-
-    dialog->addModule("kcm_ktp_accounts");
-    dialog->addModule("kcm_ktp_integration_module");
-
-    // Setup notifications menu
-    KNotifyConfigWidget *notificationWidget = new KNotifyConfigWidget(dialog);
-    notificationWidget->setApplication("ktelepathy");
-    connect(dialog, SIGNAL(accepted()),
-            notificationWidget, SLOT(save()));
-
-    connect(notificationWidget, SIGNAL(changed(bool)),
-            dialog, SLOT(enableButtonApply(bool)));
-
-    connect(dialog,SIGNAL(applyClicked()),
-            notificationWidget, SLOT(save()));
-
-    KPageWidgetItem* notificationPage = new KPageWidgetItem(notificationWidget, i18n("Notifications"));
-    notificationPage->setIcon(KIcon("preferences-desktop-notification"));
-    dialog->addPage(notificationPage);
-
-    dialog->resize(700, 640);
-    dialog->setAttribute(Qt::WA_DeleteOnClose);
-    dialog->exec();
+    KTp::SettingsKcmDialog *dialog = new KTp::SettingsKcmDialog(this);
+    dialog->addGeneralSettingsModule();
+    dialog->addNotificationsModule();
+    dialog->show();
 }
 
 void ContactListWidget::onContactListClicked(const QModelIndex& index)
@@ -496,10 +474,13 @@ void ContactListWidget::startFileTransferChannel(const Tp::AccountPtr &account,
 {
     kDebug() << "Requesting file transfer for contact" << contact->alias();
 
-    QStringList filenames = KFileDialog::getOpenFileNames(KUrl("kfiledialog:///FileTransferLastDirectory"),
-                                                          QString(),
-                                                          this,
-                                                          i18n("Choose files to send to %1", contact->alias()));
+    KFileDialog *fileDialog = new KFileDialog(KUrl("kfiledialog:///FileTransferLastDirectory"), QString(), this);
+    fileDialog->setOperationMode(KFileDialog::Opening);
+    fileDialog->setWindowTitle(i18n("Choose files to send to %1", contact->alias()));
+    fileDialog->okButton()->setText(i18n("Send"));
+    fileDialog->exec();
+    QStringList filenames = fileDialog->selectedFiles();
+    fileDialog->deleteLater();
 
     if (filenames.isEmpty()) { // User hit cancel button
         return;
@@ -521,12 +502,21 @@ void ContactListWidget::requestFileTransferChannels(const Tp::AccountPtr &accoun
     Q_EMIT actionStarted();
 }
 
-void ContactListWidget::onNewGroupModelItemsInserted(const QModelIndex& index, int start, int end)
+void ContactListWidget::onNewGroupModelItemsInserted(const QModelIndex& parentIndex, int start, int end)
 {
-    Q_UNUSED(start);
     Q_UNUSED(end);
     Q_D(ContactListWidget);
 
+    QModelIndex index;
+
+    //if the inserted item's parent is valid, it is probably the top-level item we want to expand/collapse
+    if (parentIndex.isValid()) {
+        index = parentIndex;
+    } else {
+        //if it is invalid, the inserted item may be a group added after a child, so we get the group index
+        index = model()->index(start, 0, QModelIndex());
+    }
+
     if (!index.isValid()) {
         return;
     }
